@@ -33,8 +33,22 @@
 
 #include <config.h>
 #include "softhsm2-util.h"
+#include "findslot.h"
 #include "getpw.h"
 #include "library.h"
+#include "log.h"
+#include "Configuration.h"
+#include "SimpleConfigLoader.h"
+#include "Directory.h"
+#include "MutexFactory.h"
+#include "ObjectStoreToken.h"
+#include "OSPathSep.h"
+
+#if defined(WITH_OPENSSL)
+#include "OSSLCryptoFactory.h"
+#else
+#include "BotanCryptoFactory.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,32 +58,64 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <dirent.h>
+#else
+#include <direct.h>
+#include <io.h>
 #endif
 #include <iostream>
 #include <fstream>
 
+// Initialise the one-and-only instance
+
+#ifdef HAVE_CXX11
+
+std::unique_ptr<MutexFactory> MutexFactory::instance(nullptr);
+std::unique_ptr<SecureMemoryRegistry> SecureMemoryRegistry::instance(nullptr);
+#if defined(WITH_OPENSSL)
+std::unique_ptr<OSSLCryptoFactory> OSSLCryptoFactory::instance(nullptr);
+#else
+std::unique_ptr<BotanCryptoFactory> BotanCryptoFactory::instance(nullptr);
+#endif
+
+#else
+
+std::auto_ptr<MutexFactory> MutexFactory::instance(NULL);
+std::auto_ptr<SecureMemoryRegistry> SecureMemoryRegistry::instance(NULL);
+#if defined(WITH_OPENSSL)
+std::auto_ptr<OSSLCryptoFactory> OSSLCryptoFactory::instance(NULL);
+#else
+std::auto_ptr<BotanCryptoFactory> BotanCryptoFactory::instance(NULL);
+#endif
+
+#endif
+
 // Display the usage
 void usage()
 {
 	printf("Support tool for PKCS#11\n");
 	printf("Usage: softhsm2-util [ACTION] [OPTIONS]\n");
 	printf("Action:\n");
+	printf("  --delete-token    Delete the token at a given slot.\n");
+	printf("                    Use with --token or --serial.\n");
+	printf("                    WARNING: Any content in token will be erased.\n");
 	printf("  -h                Shows this help screen.\n");
 	printf("  --help            Shows this help screen.\n");
 	printf("  --import <path>   Import a key pair from the given path.\n");
 	printf("                    The file must be in PKCS#8-format.\n");
-	printf("                    Use with --file-pin, --slot, --label, --id,\n");
-	printf("                    --no-public-key, and --pin.\n");
+	printf("                    Use with --slot or --token or --serial, --file-pin,\n");
+	printf("                    --label, --id, --no-public-key, and --pin.\n");
 	printf("  --init-token      Initialize the token at a given slot.\n");
-	printf("                    Use with --slot or --free, --label, --so-pin, and --pin.\n");
-	printf("                    WARNING: Any content in token token will be erased.\n");
+	printf("                    Use with --slot or --token or --serial or --free,\n");
+	printf("                    --label, --so-pin, and --pin.\n");
+	printf("                    WARNING: Any content in token will be erased.\n");
 	printf("  --show-slots      Display all the available slots.\n");
 	printf("  -v                Show version info.\n");
 	printf("  --version         Show version info.\n");
 	printf("Options:\n");
 	printf("  --file-pin <PIN>  Supply a PIN if the file is encrypted.\n");
 	printf("  --force           Used to override a warning.\n");
-	printf("  --free            Initialize the first free token.\n");
+	printf("  --free            Use the first free/uninitialized token.\n");
 	printf("  --id <hex>        Defines the ID of the object. Hexadecimal characters.\n");
 	printf("                    Use with --force if multiple key pairs may share\n");
 	printf("                    the same ID.\n");
@@ -77,13 +123,16 @@ void usage()
 	printf("  --module <path>   Use another PKCS#11 library than SoftHSM.\n");
 	printf("  --no-public-key   Do not import the public key.\n");
 	printf("  --pin <PIN>       The PIN for the normal user.\n");
+	printf("  --serial <number> Will use the token with a matching serial number.\n");
 	printf("  --slot <number>   The slot where the token is located.\n");
 	printf("  --so-pin <PIN>    The PIN for the Security Officer (SO).\n");
+	printf("  --token <label>   Will use the token with a matching token label.\n");
 }
 
 // Enumeration of the long options
 enum {
-	OPT_FILE_PIN = 0x100,
+	OPT_DELETE_TOKEN = 0x100,
+	OPT_FILE_PIN,
 	OPT_FORCE,
 	OPT_FREE,
 	OPT_HELP,
@@ -94,14 +143,17 @@ enum {
 	OPT_MODULE,
 	OPT_NO_PUBLIC_KEY,
 	OPT_PIN,
+	OPT_SERIAL,
 	OPT_SHOW_SLOTS,
 	OPT_SLOT,
 	OPT_SO_PIN,
+	OPT_TOKEN,
 	OPT_VERSION
 };
 
 // Text representation of the long options
 static const struct option long_options[] = {
+	{ "delete-token",    0, NULL, OPT_DELETE_TOKEN },
 	{ "file-pin",        1, NULL, OPT_FILE_PIN },
 	{ "force",           0, NULL, OPT_FORCE },
 	{ "free",            0, NULL, OPT_FREE },
@@ -113,9 +165,11 @@ static const struct option long_options[] = {
 	{ "module",          1, NULL, OPT_MODULE },
 	{ "no-public-key",   0, NULL, OPT_NO_PUBLIC_KEY },
 	{ "pin",             1, NULL, OPT_PIN },
+	{ "serial",          1, NULL, OPT_SERIAL },
 	{ "show-slots",      0, NULL, OPT_SHOW_SLOTS },
 	{ "slot",            1, NULL, OPT_SLOT },
 	{ "so-pin",          1, NULL, OPT_SO_PIN },
+	{ "token",           1, NULL, OPT_TOKEN },
 	{ "version",         0, NULL, OPT_VERSION },
 	{ NULL,              0, NULL, 0 }
 };
@@ -129,7 +183,6 @@ int main(int argc, char* argv[])
 	int opt;
 
 	char* inPath = NULL;
-	//char* outPath = NULL;
 	char* soPIN = NULL;
 	char* userPIN = NULL;
 	char* filePIN = NULL;
@@ -137,17 +190,21 @@ int main(int argc, char* argv[])
 	char* module = NULL;
 	char* objectID = NULL;
 	char* slot = NULL;
+	char* serial = NULL;
+	char* token = NULL;
 	char* errMsg = NULL;
 	int forceExec = 0;
-	int freeToken = 0;
+	bool freeToken = false;
 	int noPublicKey = 0;
 
 	int doInitToken = 0;
 	int doShowSlots = 0;
 	int doImport = 0;
-	//int doExport = 0;
+	int doDeleteToken = 0;
 	int action = 0;
+	bool needP11 = false;
 	int rv = 0;
+	CK_SLOT_ID slotID = 0;
 
 	moduleHandle = NULL;
 	p11 = NULL;
@@ -159,15 +216,22 @@ int main(int argc, char* argv[])
 			case OPT_SHOW_SLOTS:
 				doShowSlots = 1;
 				action++;
+				needP11 = true;
 				break;
 			case OPT_INIT_TOKEN:
 				doInitToken = 1;
 				action++;
+				needP11 = true;
 				break;
 			case OPT_IMPORT:
 				doImport = 1;
 				action++;
 				inPath = optarg;
+				needP11 = true;
+				break;
+			case OPT_DELETE_TOKEN:
+				doDeleteToken = 1;
+				action++;
 				break;
 			case OPT_SLOT:
 				slot = optarg;
@@ -175,6 +239,12 @@ int main(int argc, char* argv[])
 			case OPT_LABEL:
 				label = optarg;
 				break;
+			case OPT_SERIAL:
+				serial = optarg;
+				break;
+			case OPT_TOKEN:
+				token = optarg;
+				break;
 			case OPT_MODULE:
 				module = optarg;
 				break;
@@ -197,7 +267,7 @@ int main(int argc, char* argv[])
 				forceExec = 1;
 				break;
 			case OPT_FREE:
-				freeToken = 1;
+				freeToken = true;
 				break;
 			case OPT_VERSION:
 			case 'v':
@@ -214,11 +284,13 @@ int main(int argc, char* argv[])
 	}
 
 	// No action given, display the usage.
-	if (!action)
+	if (action != 1)
 	{
 		usage();
+		exit(1);
 	}
-	else
+
+	if (needP11)
 	{
 		// Get a pointer to the function list for PKCS#11 library
 		CK_C_GetFunctionList pGetFunctionList = loadLibrary(module, &moduleHandle, &errMsg);
@@ -243,24 +315,47 @@ int main(int argc, char* argv[])
 	// We should create the token.
 	if (doInitToken)
 	{
-		rv = initToken(slot, freeToken, label, soPIN, userPIN);
+		// Get the slotID
+		rv = findSlot(slot, serial, token, freeToken, slotID);
+		if (!rv)
+		{
+			rv = initToken(slotID, label, soPIN, userPIN);
+		}
 	}
 
 	// Show all available slots
-	if (doShowSlots)
+	if (!rv && doShowSlots)
 	{
 		rv = showSlots();
 	}
 
 	// Import a key pair from the given path
-	if (doImport)
+	if (!rv && doImport)
 	{
-		rv = importKeyPair(inPath, filePIN, slot, userPIN, label, objectID,
-					forceExec, noPublicKey);
+		// Get the slotID
+		rv = findSlot(slot, serial, token, slotID);
+		if (!rv)
+		{
+			rv = importKeyPair(inPath, filePIN, slotID, userPIN, label, objectID,
+						forceExec, noPublicKey);
+		}
+	}
+
+	// We should delete the token.
+	if (!rv && doDeleteToken)
+	{
+		if (deleteToken(serial, token))
+		{
+			rv = 0;
+		}
+		else
+		{
+			rv = 1;
+		}
 	}
 
 	// Finalize the library
-	if (action)
+	if (needP11)
 	{
 		p11->C_Finalize(NULL_PTR);
 		unloadLibrary(moduleHandle);
@@ -270,24 +365,11 @@ int main(int argc, char* argv[])
 }
 
 // Initialize the token
-int initToken(char* slot, int freeToken, char* label, char* soPIN, char* userPIN)
+int initToken(CK_SLOT_ID slotID, char* label, char* soPIN, char* userPIN)
 {
 	char so_pin_copy[MAX_PIN_LEN+1];
 	char user_pin_copy[MAX_PIN_LEN+1];
 
-	if (slot != NULL && freeToken == 1)
-	{
-		fprintf(stderr, "ERROR: Select --slot <number> or --free\n");
-		return 1;
-	}
-
-	if (slot == NULL && freeToken != 1)
-	{
-		fprintf(stderr, "ERROR: A slot number must be supplied. "
-				"Use --slot <number> or --free\n");
-		return 1;
-	}
-
 	if (label == NULL)
 	{
 		fprintf(stderr, "ERROR: A label for the token must be supplied. "
@@ -303,33 +385,22 @@ int initToken(char* slot, int freeToken, char* label, char* soPIN, char* userPIN
 	}
 
 	// Get the passwords
-	getPW(soPIN, so_pin_copy, CKU_SO);
-	getPW(userPIN, user_pin_copy, CKU_USER);
+	if (getPW(soPIN, so_pin_copy, CKU_SO) != 0)
+	{
+		fprintf(stderr, "ERROR: Could not get SO PIN\n");
+		return 1;
+	}
+	if (getPW(userPIN, user_pin_copy, CKU_USER) != 0)
+	{
+		fprintf(stderr, "ERROR: Could not get user PIN\n");
+		return 1;
+	}
 
 	// Load the variables
-	CK_SLOT_ID slotID = 0;
 	CK_UTF8CHAR paddedLabel[32];
 	memset(paddedLabel, ' ', sizeof(paddedLabel));
 	memcpy(paddedLabel, label, strlen(label));
 
-	if (slot != NULL)
-	{
-		int slotNumber = atoi(slot);
-		if (slotNumber < 0)
-		{
-			fprintf(stderr, "ERROR: The slot number is negative.\n");
-			return 1;
-		}
-
-		slotID = slotNumber;
-	}
-	else
-	{
-		CK_SLOT_ID slotNumber = 0;
-		if (getFirstFreeToken(slotNumber) != 0) return 1;
-		slotID = slotNumber;
-	}
-
 	CK_RV rv = p11->C_InitToken(slotID, (CK_UTF8CHAR_PTR)so_pin_copy, strlen(so_pin_copy), paddedLabel);
 
 	switch (rv)
@@ -383,58 +454,396 @@ int initToken(char* slot, int freeToken, char* label, char* soPIN, char* userPIN
 	return 0;
 }
 
-// Search and return the first free token
-int getFirstFreeToken(CK_SLOT_ID &slot)
+// Delete the token
+bool deleteToken(char* serial, char* token)
 {
-	CK_ULONG ulSlotCount;
-	CK_RV rv = p11->C_GetSlotList(CK_TRUE, NULL_PTR, &ulSlotCount);
-	if (rv != CKR_OK)
+	if (serial == NULL && token == NULL)
 	{
-		fprintf(stderr, "ERROR: Could not get the number of slots.\n");
-		return -1;
+		fprintf(stderr, "ERROR: A token must be supplied. "
+				"Use --serial <serial> or --token <label>\n");
+		return false;
 	}
 
-	CK_SLOT_ID_PTR pSlotList = (CK_SLOT_ID_PTR) malloc(ulSlotCount*sizeof(CK_SLOT_ID));
-	if (!pSlotList)
+	// Initialize the SoftHSM internal functions
+	if (!initSoftHSM())
 	{
-		fprintf(stderr, "ERROR: Could not allocate memory.\n");
-		return -1;
+		finalizeSoftHSM();
+		return false;
 	}
 
-	rv = p11->C_GetSlotList(CK_FALSE, pSlotList, &ulSlotCount);
-	if (rv != CKR_OK)
-	{
-		fprintf(stderr, "ERROR: Could not get the slot list.\n");
-		free(pSlotList);
-		return -1;
-	}
+	bool rv = true;
+	std::string basedir = Configuration::i()->getString("directories.tokendir", DEFAULT_TOKENDIR);
+	std::string tokendir;
 
-	for (CK_ULONG i = 0; i < ulSlotCount; i++)
-	{
-		CK_TOKEN_INFO tokenInfo;
+	rv = findTokenDirectory(basedir, tokendir, serial, token);
 
-		rv = p11->C_GetTokenInfo(pSlotList[i], &tokenInfo);
-		if (rv != CKR_OK)
+	if (rv)
+	{
+		std::string fulldir = basedir;
+		if (fulldir.find_last_of(OS_PATHSEP) != (fulldir.size()-1))
 		{
-			fprintf(stderr, "ERROR: Could not get info about the token in slot %lu.\n",
-				pSlotList[i]);
-			free(pSlotList);
-			return -1;
+			fulldir += OS_PATHSEP + tokendir;
+		}
+		else
+		{
+			fulldir += tokendir;
 		}
 
-		if ((tokenInfo.flags & CKF_TOKEN_INITIALIZED) == 0)
+		rv = rmdir(fulldir);
+		if (rv)
 		{
-			printf("Token %lu is free.\n", pSlotList[i]);
-			slot = pSlotList[i];
-			free(pSlotList);
-			return 0;
+			printf("The token (%s) has been deleted.\n", fulldir.c_str());
 		}
 	}
 
-	free(pSlotList);
+	finalizeSoftHSM();
 
-	fprintf(stderr, "ERROR: Could not find the first free token.\n");
-	return -1;
+	return rv;
+}
+
+bool initSoftHSM()
+{
+	// Not using threading
+	MutexFactory::i()->disable();
+
+	// Initiate SecureMemoryRegistry
+	if (SecureMemoryRegistry::i() == NULL)
+	{
+		fprintf(stderr, "ERROR: Could not initiate SecureMemoryRegistry.\n");
+		return false;
+	}
+
+	// Build the CryptoFactory
+	if (CryptoFactory::i() == NULL)
+	{
+		fprintf(stderr, "ERROR: Could not initiate CryptoFactory.\n");
+		return false;
+	}
+
+#ifdef WITH_FIPS
+	// Check the FIPS status
+	if (!CryptoFactory::i()->getFipsSelfTestStatus())
+	{
+		fprintf(stderr, "ERROR: FIPS self test failed.\n");
+		return false;
+	}
+#endif
+
+	// Load the configuration
+	if (!Configuration::i()->reload(SimpleConfigLoader::i()))
+	{
+		fprintf(stderr, "ERROR: Could not load the SoftHSM configuration.\n");
+		return false;
+	}
+
+	// Configure the log level
+	if (!setLogLevel(Configuration::i()->getString("log.level", DEFAULT_LOG_LEVEL)))
+	{
+		fprintf(stderr, "ERROR: Could not configure the log level.\n");
+		return false;
+	}
+
+	// Configure object store storage backend used by all tokens.
+	if (!ObjectStoreToken::selectBackend(Configuration::i()->getString("objectstore.backend", DEFAULT_OBJECTSTORE_BACKEND)))
+	{
+		fprintf(stderr, "ERROR: Could not select token backend.\n");
+		return false;
+	}
+
+	return true;
+}
+
+void finalizeSoftHSM()
+{
+	CryptoFactory::reset();
+	SecureMemoryRegistry::reset();
+}
+
+// Find the token directory
+bool findTokenDirectory(std::string basedir, std::string& tokendir, char* serial, char* label)
+{
+	if (serial == NULL && label == NULL)
+	{
+		return false;
+	}
+
+	// Load the variables
+	CK_UTF8CHAR paddedSerial[16];
+	CK_UTF8CHAR paddedLabel[32];
+	if (serial != NULL)
+	{
+		size_t inSize = strlen(serial);
+		size_t outSize = sizeof(paddedSerial);
+		if (inSize > outSize)
+		{
+			fprintf(stderr, "ERROR: --serial is too long.\n");
+			return false;
+		}
+		memset(paddedSerial, ' ', outSize);
+		memcpy(paddedSerial, serial, inSize);
+	}
+	if (label != NULL)
+	{
+		size_t inSize = strlen(label);
+		size_t outSize = sizeof(paddedLabel);
+		if (inSize > outSize)
+		{
+			fprintf(stderr, "ERROR: --token is too long.\n");
+			return false;
+		}
+		memset(paddedLabel, ' ', outSize);
+		memcpy(paddedLabel, label, inSize);
+	}
+
+	// Find all tokens in the specified path
+	Directory storeDir(basedir);
+
+	if (!storeDir.isValid())
+	{
+		fprintf(stderr, "Failed to enumerate object store in %s", basedir.c_str());
+
+		return false;
+	}
+
+	// Assume that all subdirectories are tokens
+	std::vector<std::string> dirs = storeDir.getSubDirs();
+
+	ByteString tokenLabel;
+	ByteString tokenSerial;
+	CK_UTF8CHAR paddedTokenSerial[16];
+	CK_UTF8CHAR paddedTokenLabel[32];
+	size_t counter = 0;
+	for (std::vector<std::string>::iterator i = dirs.begin(); i != dirs.end(); i++)
+	{
+		memset(paddedTokenSerial, ' ', sizeof(paddedTokenSerial));
+		memset(paddedTokenLabel, ' ', sizeof(paddedTokenLabel));
+
+		// Create a token instance
+		ObjectStoreToken* token = ObjectStoreToken::accessToken(basedir, *i);
+
+		if (!token->isValid())
+		{
+			delete token;
+			continue;
+		}
+
+		if (token->getTokenLabel(tokenLabel) && tokenLabel.size() <= sizeof(paddedTokenLabel))
+		{
+			strncpy((char*) paddedTokenLabel, (char*) tokenLabel.byte_str(), tokenLabel.size());
+		}
+		if (token->getTokenSerial(tokenSerial) && tokenSerial.size() <= sizeof(paddedTokenSerial))
+		{
+			strncpy((char*) paddedTokenSerial, (char*) tokenSerial.byte_str(), tokenSerial.size());
+		}
+
+		if (serial != NULL && label == NULL &&
+			memcmp(paddedTokenSerial, paddedSerial, sizeof(paddedSerial)) == 0)
+		{
+			printf("Found token (%s) with matching serial.\n", i->c_str());
+			tokendir = i->c_str();
+			counter++;
+		}
+		if (serial == NULL && label != NULL &&
+			memcmp(paddedTokenLabel, paddedLabel, sizeof(paddedLabel)) == 0)
+		{
+			printf("Found token (%s) with matching token label.\n", i->c_str());
+			tokendir = i->c_str();
+			counter++;
+		}
+		if (serial != NULL && label != NULL &&
+			memcmp(paddedTokenSerial, paddedSerial, sizeof(paddedSerial)) == 0 &&
+			memcmp(paddedTokenLabel, paddedLabel, sizeof(paddedLabel)) == 0)
+		{
+			printf("Found token (%s) with matching serial and token label.\n", i->c_str());
+			tokendir = i->c_str();
+			counter++;
+		}
+
+		delete token;
+	}
+
+	if (counter == 1) return true;
+	if (counter > 1)
+	{
+		fprintf(stderr, "ERROR: Found multiple matching tokens.\n");
+		return false;
+	}
+
+	fprintf(stderr, "ERROR: Could not find a token using --serial or --token.\n");
+	return false;
+}
+
+
+// Delete a directory
+bool rmdir(std::string path)
+{
+	bool rv = true;
+
+#ifndef _WIN32
+	// Enumerate the directory
+	DIR* dir = opendir(path.c_str());
+
+	if (dir == NULL)
+	{
+		fprintf(stderr, "ERROR: Failed to open directory %s", path.c_str());
+		return false;
+	}
+
+	// Enumerate the directory
+	struct dirent* entry = NULL;
+
+	while ((entry = readdir(dir)) != NULL)
+	{
+		bool handled = false;
+
+		// Check if this is the . or .. entry
+		if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, ".."))
+		{
+			continue;
+		}
+
+		// Convert the name of the entry to a C++ string
+		std::string name(entry->d_name);
+		std::string fullPath = path + OS_PATHSEP + name;
+
+#if defined(_DIRENT_HAVE_D_TYPE) && defined(_BSD_SOURCE)
+		// Determine the type of the entry
+		switch(entry->d_type)
+		{
+			case DT_DIR:
+				// This is a directory
+				rv = rmdir(fullPath);
+				handled = true;
+				break;
+			case DT_REG:
+				// This is a regular file
+				rv = rm(fullPath);
+				handled = true;
+				break;
+			default:
+				break;
+		}
+#endif
+
+		if (rv == false)
+			break;
+
+		if (!handled)
+		{
+			// The entry type has to be determined using lstat
+			struct stat entryStatus;
+
+			if (!lstat(fullPath.c_str(), &entryStatus))
+			{
+				if (S_ISDIR(entryStatus.st_mode))
+				{
+					// This is a directory
+					rv = rmdir(fullPath);
+				}
+				else if (S_ISREG(entryStatus.st_mode))
+				{
+					// This is a regular file
+					rv = rm(fullPath);
+				}
+			}
+
+			if (rv == false)
+				break;
+		}
+	}
+
+	// Close the directory
+	closedir(dir);
+#else
+	// Enumerate the directory
+	std::string pattern;
+	intptr_t h;
+	struct _finddata_t fi;
+
+	if ((path.back() == '/') || (path.back() == '\\'))
+		pattern = path + "*";
+	else
+		pattern = path + "/*";
+	memset(&fi, 0, sizeof(fi));
+	h = _findfirst(pattern.c_str(), &fi);
+	if (h == -1)
+	{
+		// empty directory
+		if (errno == ENOENT)
+			goto finished;
+
+		fprintf(stderr, "ERROR: Failed to open directory %s", path.c_str());
+
+		return false;
+	}
+
+	// scan files & subdirs
+	do
+	{
+		// Check if this is the . or .. entry
+		if (!strcmp(fi.name, ".") || !strcmp(fi.name, ".."))
+			continue;
+
+		std::string fullPath = path + OS_PATHSEP + fi.name;
+		if ((fi.attrib & _A_SUBDIR) == 0)
+		{
+			// This is a regular file
+			rv = rm(fullPath);
+		}
+		else
+		{
+			// This is a directory
+			rv = rmdir(fullPath);
+		}
+
+		memset(&fi, 0, sizeof(fi));
+
+		if (rv == false)
+			break;
+	} while (_findnext(h, &fi) == 0);
+
+	(void) _findclose(h);
+
+    finished:
+#endif
+
+	if (rv == false)
+		return false;
+
+	int result;
+#ifndef _WIN32
+	result = ::rmdir(path.c_str());
+#else
+	result = _rmdir(path.c_str());
+#endif
+
+	if (result != 0)
+	{
+		fprintf(stderr, "ERROR: Could not delete the directory: %s\n", path.c_str());
+		return false;
+	}
+
+	return true;
+}
+
+// Delete a file
+bool rm(std::string path)
+{
+	int result;
+
+#ifndef _WIN32
+	result = ::remove(path.c_str());
+#else
+	result = _unlink(path.c_str());
+#endif
+
+	if (result != 0)
+	{
+		fprintf(stderr, "ERROR: Could not delete the file: %s\n", path.c_str());
+		return false;
+	}
+
+	return true;
 }
 
 // Show what slots are available
@@ -544,7 +953,7 @@ int importKeyPair
 (
 	char* filePath,
 	char* filePIN,
-	char* slot,
+	CK_SLOT_ID slotID,
 	char* userPIN,
 	char* label,
 	char* objectID,
@@ -554,13 +963,6 @@ int importKeyPair
 {
 	char user_pin_copy[MAX_PIN_LEN+1];
 
-	if (slot == NULL)
-	{
-		fprintf(stderr, "ERROR: A slot number must be supplied. "
-				"Use --slot <number>\n");
-		return 1;
-	}
-
 	if (label == NULL)
 	{
 		fprintf(stderr, "ERROR: A label for the object must be supplied. "
@@ -583,7 +985,6 @@ int importKeyPair
 		return 1;
 	}
 
-	CK_SLOT_ID slotID = atoi(slot);
 	CK_SESSION_HANDLE hSession;
 	CK_RV rv = p11->C_OpenSession(slotID, CKF_SERIAL_SESSION | CKF_RW_SESSION,
 					NULL_PTR, NULL_PTR, &hSession);
@@ -602,7 +1003,12 @@ int importKeyPair
 	}
 
 	// Get the password
-	getPW(userPIN, user_pin_copy, CKU_USER);
+	if (getPW(userPIN, user_pin_copy, CKU_USER) != 0)
+	{
+		fprintf(stderr, "ERROR: Could not get user PIN\n");
+		free(objID);
+		return 1;
+	}
 
 	rv = p11->C_Login(hSession, CKU_USER, (CK_UTF8CHAR_PTR)user_pin_copy, strlen(user_pin_copy));
 	if (rv != CKR_OK)
