@@ -35,9 +35,7 @@
 #include <config.h>
 #include <stdlib.h>
 #include <string.h>
-#include <cppunit/extensions/HelperMacros.h>
 #include "DeriveTests.h"
-#include "testconfig.h"
 
 // CKA_TOKEN
 const CK_BBOOL ON_TOKEN = CK_TRUE;
@@ -50,51 +48,6 @@ const CK_BBOOL IS_PUBLIC = CK_FALSE;
 
 CPPUNIT_TEST_SUITE_REGISTRATION(DeriveTests);
 
-void DeriveTests::setUp()
-{
-//    printf("\nDeriveTests\n");
-
-#ifndef _WIN32
-	setenv("SOFTHSM2_CONF", "./softhsm2.conf", 1);
-#else
-	setenv("SOFTHSM2_CONF", ".\\softhsm2.conf", 1);
-#endif
-
-	CK_RV rv;
-	CK_UTF8CHAR pin[] = SLOT_0_USER1_PIN;
-	CK_ULONG pinLength = sizeof(pin) - 1;
-	CK_UTF8CHAR sopin[] = SLOT_0_SO1_PIN;
-	CK_ULONG sopinLength = sizeof(sopin) - 1;
-	CK_SESSION_HANDLE hSession;
-
-	CK_UTF8CHAR label[32];
-	memset(label, ' ', 32);
-	memcpy(label, "token1", strlen("token1"));
-
-	// (Re)initialize the token
-	rv = C_Initialize(NULL_PTR);
-	CPPUNIT_ASSERT(rv == CKR_OK);
-	rv = C_InitToken(SLOT_INIT_TOKEN, sopin,sopinLength, label);
-	CPPUNIT_ASSERT(rv == CKR_OK);
-
-	// Open session
-	rv = C_OpenSession(SLOT_INIT_TOKEN, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSession);
-	CPPUNIT_ASSERT(rv == CKR_OK);
-
-	// Login SO
-	rv = C_Login(hSession,CKU_SO, sopin, sopinLength);
-	CPPUNIT_ASSERT(rv == CKR_OK);
-
-	// Initialize the user pin
-	rv = C_InitPIN(hSession, pin, pinLength);
-	CPPUNIT_ASSERT(rv == CKR_OK);
-}
-
-void DeriveTests::tearDown()
-{
-	C_Finalize(NULL_PTR);
-}
-
 CK_RV DeriveTests::generateDhKeyPair(CK_SESSION_HANDLE hSession, CK_BBOOL bTokenPuk, CK_BBOOL bPrivatePuk, CK_BBOOL bTokenPrk, CK_BBOOL bPrivatePrk, CK_OBJECT_HANDLE &hPuk, CK_OBJECT_HANDLE &hPrk)
 {
 	CK_MECHANISM mechanism = { CKM_DH_PKCS_KEY_PAIR_GEN, NULL_PTR, 0 };
@@ -133,10 +86,10 @@ CK_RV DeriveTests::generateDhKeyPair(CK_SESSION_HANDLE hSession, CK_BBOOL bToken
 
 	hPuk = CK_INVALID_HANDLE;
 	hPrk = CK_INVALID_HANDLE;
-	return C_GenerateKeyPair(hSession, &mechanism,
+	return CRYPTOKI_F_PTR( C_GenerateKeyPair(hSession, &mechanism,
 			pukAttribs, sizeof(pukAttribs)/sizeof(CK_ATTRIBUTE),
 			prkAttribs, sizeof(prkAttribs)/sizeof(CK_ATTRIBUTE),
-			&hPuk, &hPrk);
+			&hPuk, &hPrk) );
 }
 
 CK_RV DeriveTests::generateAesKey(CK_SESSION_HANDLE hSession, CK_BBOOL bToken, CK_BBOOL bPrivate, CK_OBJECT_HANDLE &hKey)
@@ -154,9 +107,9 @@ CK_RV DeriveTests::generateAesKey(CK_SESSION_HANDLE hSession, CK_BBOOL bToken, C
 	};
 
 	hKey = CK_INVALID_HANDLE;
-	return C_GenerateKey(hSession, &mechanism,
+	return CRYPTOKI_F_PTR( C_GenerateKey(hSession, &mechanism,
 			     keyAttribs, sizeof(keyAttribs)/sizeof(CK_ATTRIBUTE),
-			     &hKey);
+			     &hKey) );
 }
 
 #ifndef WITH_FIPS
@@ -173,9 +126,9 @@ CK_RV DeriveTests::generateDesKey(CK_SESSION_HANDLE hSession, CK_BBOOL bToken, C
 	};
 
 	hKey = CK_INVALID_HANDLE;
-	return C_GenerateKey(hSession, &mechanism,
+	return CRYPTOKI_F_PTR( C_GenerateKey(hSession, &mechanism,
 			     keyAttribs, sizeof(keyAttribs)/sizeof(CK_ATTRIBUTE),
-			     &hKey);
+			     &hKey) );
 }
 #endif
 
@@ -192,9 +145,9 @@ CK_RV DeriveTests::generateDes2Key(CK_SESSION_HANDLE hSession, CK_BBOOL bToken,
 	};
 
 	hKey = CK_INVALID_HANDLE;
-	return C_GenerateKey(hSession, &mechanism,
+	return CRYPTOKI_F_PTR( C_GenerateKey(hSession, &mechanism,
 			     keyAttribs, sizeof(keyAttribs)/sizeof(CK_ATTRIBUTE),
-			     &hKey);
+			     &hKey) );
 }
 
 CK_RV DeriveTests::generateDes3Key(CK_SESSION_HANDLE hSession, CK_BBOOL bToken, CK_BBOOL bPrivate, CK_OBJECT_HANDLE &hKey)
@@ -210,18 +163,18 @@ CK_RV DeriveTests::generateDes3Key(CK_SESSION_HANDLE hSession, CK_BBOOL bToken,
 	};
 
 	hKey = CK_INVALID_HANDLE;
-	return C_GenerateKey(hSession, &mechanism,
+	return CRYPTOKI_F_PTR( C_GenerateKey(hSession, &mechanism,
 			     keyAttribs, sizeof(keyAttribs)/sizeof(CK_ATTRIBUTE),
-			     &hKey);
+			     &hKey) );
 }
 
 void DeriveTests::dhDerive(CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hPublicKey, CK_OBJECT_HANDLE hPrivateKey, CK_OBJECT_HANDLE &hKey)
 {
 	CK_ATTRIBUTE valAttrib = { CKA_VALUE, NULL_PTR, 0 };
-	CK_RV rv = C_GetAttributeValue(hSession, hPublicKey, &valAttrib, 1);
+	CK_RV rv = CRYPTOKI_F_PTR( C_GetAttributeValue(hSession, hPublicKey, &valAttrib, 1) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
 	valAttrib.pValue = (CK_BYTE_PTR)malloc(valAttrib.ulValueLen);
-	rv = C_GetAttributeValue(hSession, hPublicKey, &valAttrib, 1);
+	rv = CRYPTOKI_F_PTR( C_GetAttributeValue(hSession, hPublicKey, &valAttrib, 1) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
 	CK_MECHANISM mechanism = { CKM_DH_PKCS_DERIVE, NULL_PTR, 0 };
 	mechanism.pParameter = valAttrib.pValue;
@@ -241,60 +194,70 @@ void DeriveTests::dhDerive(CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hPublicK
 	};
 
 	hKey = CK_INVALID_HANDLE;
-	rv = C_DeriveKey(hSession, &mechanism, hPrivateKey,
+	rv = CRYPTOKI_F_PTR( C_DeriveKey(hSession, &mechanism, hPrivateKey,
 			 keyAttribs, sizeof(keyAttribs)/sizeof(CK_ATTRIBUTE),
-			 &hKey);
+			 &hKey) );
 	free(valAttrib.pValue);
 	CPPUNIT_ASSERT(rv == CKR_OK);
 }
 
 bool DeriveTests::compareSecret(CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hKey1, CK_OBJECT_HANDLE hKey2)
 {
-	CK_ATTRIBUTE valAttrib = { CKA_VALUE, NULL_PTR, 0 };
+	CK_ATTRIBUTE keyAttribs[] = {
+		{ CKA_VALUE, NULL_PTR, 0 },
+		{ CKA_CHECK_VALUE, NULL_PTR, 0 }
+	};
 	CK_BYTE val1[128];
-	valAttrib.pValue = val1;
-	valAttrib.ulValueLen = sizeof(val1);
-	CK_RV rv = C_GetAttributeValue(hSession, hKey1, &valAttrib, 1);
+	CK_BYTE check1[3];
+	keyAttribs[0].pValue = val1;
+	keyAttribs[0].ulValueLen = sizeof(val1);
+	keyAttribs[1].pValue = check1;
+	keyAttribs[1].ulValueLen = sizeof(check1);
+	CK_RV rv = CRYPTOKI_F_PTR( C_GetAttributeValue(hSession, hKey1, keyAttribs, 2) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	CPPUNIT_ASSERT(valAttrib.ulValueLen == 100);
+	CPPUNIT_ASSERT(keyAttribs[0].ulValueLen == 100);
+	CPPUNIT_ASSERT(keyAttribs[1].ulValueLen == 3);
 	CK_BYTE val2[128];
-	valAttrib.pValue = val2;
-	valAttrib.ulValueLen = sizeof(val2);
-	rv = C_GetAttributeValue(hSession, hKey2, &valAttrib, 1);
+	CK_BYTE check2[3];
+	keyAttribs[0].pValue = val2;
+	keyAttribs[0].ulValueLen = sizeof(val2);
+	keyAttribs[1].pValue = check2;
+	keyAttribs[1].ulValueLen = sizeof(check2);
+	rv = CRYPTOKI_F_PTR( C_GetAttributeValue(hSession, hKey2, keyAttribs, 2) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
-	CPPUNIT_ASSERT(valAttrib.ulValueLen == 100);
-	return memcmp(val1, val2, 100) == 0;
+	CPPUNIT_ASSERT(keyAttribs[0].ulValueLen == 100);
+	CPPUNIT_ASSERT(keyAttribs[1].ulValueLen == 3);
+	return memcmp(val1, val2, 100) == 0 &&
+	       memcmp(check1, check2, 3) == 0;
 }
 
 void DeriveTests::testDhDerive()
 {
 	CK_RV rv;
-	CK_UTF8CHAR pin[] = SLOT_0_USER1_PIN;
-	CK_ULONG pinLength = sizeof(pin) - 1;
 	CK_SESSION_HANDLE hSessionRO;
 	CK_SESSION_HANDLE hSessionRW;
 
 	// Just make sure that we finalize any previous tests
-	C_Finalize(NULL_PTR);
+	CRYPTOKI_F_PTR( C_Finalize(NULL_PTR) );
 
 	// Open read-only session on when the token is not initialized should fail
-	rv = C_OpenSession(SLOT_INIT_TOKEN, CKF_SERIAL_SESSION, NULL_PTR, NULL_PTR, &hSessionRO);
+	rv = CRYPTOKI_F_PTR( C_OpenSession(m_initializedTokenSlotID, CKF_SERIAL_SESSION, NULL_PTR, NULL_PTR, &hSessionRO) );
 	CPPUNIT_ASSERT(rv == CKR_CRYPTOKI_NOT_INITIALIZED);
 
 	// Initialize the library and start the test.
-	rv = C_Initialize(NULL_PTR);
+	rv = CRYPTOKI_F_PTR( C_Initialize(NULL_PTR) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
 
 	// Open read-only session
-	rv = C_OpenSession(SLOT_INIT_TOKEN, CKF_SERIAL_SESSION, NULL_PTR, NULL_PTR, &hSessionRO);
+	rv = CRYPTOKI_F_PTR( C_OpenSession(m_initializedTokenSlotID, CKF_SERIAL_SESSION, NULL_PTR, NULL_PTR, &hSessionRO) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
 
 	// Open read-write session
-	rv = C_OpenSession(SLOT_INIT_TOKEN, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSessionRW);
+	rv = CRYPTOKI_F_PTR( C_OpenSession(m_initializedTokenSlotID, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSessionRW) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
 
 	// Login USER into the sessions so we can create a private objects
-	rv = C_Login(hSessionRO,CKU_USER,pin,pinLength);
+	rv = CRYPTOKI_F_PTR( C_Login(hSessionRO,CKU_USER,m_userPin1,m_userPin1Length) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
 
 	// Public Session keys
@@ -424,18 +387,29 @@ void DeriveTests::symDerive(CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hKey, C
 	hDerive = CK_INVALID_HANDLE;
 	if (secLen > 0)
 	{
-		rv = C_DeriveKey(hSession, &mechanism, hKey,
+		rv = CRYPTOKI_F_PTR( C_DeriveKey(hSession, &mechanism, hKey,
 				 keyAttribs, sizeof(keyAttribs)/sizeof(CK_ATTRIBUTE),
-				 &hDerive);
+				 &hDerive) );
 	}
 	else
 	{
-		rv = C_DeriveKey(hSession, &mechanism, hKey,
+		rv = CRYPTOKI_F_PTR( C_DeriveKey(hSession, &mechanism, hKey,
 				 keyAttribs, sizeof(keyAttribs)/sizeof(CK_ATTRIBUTE) - 1,
-				 &hDerive);
+				 &hDerive) );
 	}
 	CPPUNIT_ASSERT(rv == CKR_OK);
 
+	// Check that KCV has been set
+	CK_ATTRIBUTE checkAttribs[] = {
+		{ CKA_CHECK_VALUE, NULL_PTR, 0 }
+	};
+	CK_BYTE check[3];
+	checkAttribs[0].pValue = check;
+	checkAttribs[0].ulValueLen = sizeof(check);
+	rv = CRYPTOKI_F_PTR( C_GetAttributeValue(hSession, hDerive, checkAttribs, 1) );
+	CPPUNIT_ASSERT(rv == CKR_OK);
+	CPPUNIT_ASSERT(checkAttribs[0].ulValueLen == 3);
+
 	if (keyType == CKK_GENERIC_SECRET) return;
 
 	CK_BYTE cipherText[300];
@@ -443,18 +417,18 @@ void DeriveTests::symDerive(CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hKey, C
 	CK_BYTE recoveredText[300];
 	CK_ULONG ulRecoveredTextLen;
 
-	rv = C_EncryptInit(hSession,&mechEncrypt,hDerive);
+	rv = CRYPTOKI_F_PTR( C_EncryptInit(hSession,&mechEncrypt,hDerive) );
 	CPPUNIT_ASSERT(rv==CKR_OK);
 
 	ulCipherTextLen = sizeof(cipherText);
-	rv = C_Encrypt(hSession,data,sizeof(data),cipherText,&ulCipherTextLen);
+	rv = CRYPTOKI_F_PTR( C_Encrypt(hSession,data,sizeof(data),cipherText,&ulCipherTextLen) );
 	CPPUNIT_ASSERT(rv==CKR_OK);
 
-	rv = C_DecryptInit(hSession,&mechEncrypt,hDerive);
+	rv = CRYPTOKI_F_PTR( C_DecryptInit(hSession,&mechEncrypt,hDerive) );
 	CPPUNIT_ASSERT(rv==CKR_OK);
 
 	ulRecoveredTextLen = sizeof(recoveredText);
-	rv = C_Decrypt(hSession,cipherText,ulCipherTextLen,recoveredText,&ulRecoveredTextLen);
+	rv = CRYPTOKI_F_PTR( C_Decrypt(hSession,cipherText,ulCipherTextLen,recoveredText,&ulRecoveredTextLen) );
 	CPPUNIT_ASSERT(rv==CKR_OK);
 	CPPUNIT_ASSERT(ulRecoveredTextLen==sizeof(data));
 
@@ -464,32 +438,30 @@ void DeriveTests::symDerive(CK_SESSION_HANDLE hSession, CK_OBJECT_HANDLE hKey, C
 void DeriveTests::testSymDerive()
 {
 	CK_RV rv;
-	CK_UTF8CHAR pin[] = SLOT_0_USER1_PIN;
-	CK_ULONG pinLength = sizeof(pin) - 1;
 	CK_SESSION_HANDLE hSessionRO;
 	CK_SESSION_HANDLE hSessionRW;
 
 	// Just make sure that we finalize any previous tests
-	C_Finalize(NULL_PTR);
+	CRYPTOKI_F_PTR( C_Finalize(NULL_PTR) );
 
 	// Open read-only session on when the token is not initialized should fail
-	rv = C_OpenSession(SLOT_INIT_TOKEN, CKF_SERIAL_SESSION, NULL_PTR, NULL_PTR, &hSessionRO);
+	rv = CRYPTOKI_F_PTR( C_OpenSession(m_initializedTokenSlotID, CKF_SERIAL_SESSION, NULL_PTR, NULL_PTR, &hSessionRO) );
 	CPPUNIT_ASSERT(rv == CKR_CRYPTOKI_NOT_INITIALIZED);
 
 	// Initialize the library and start the test.
-	rv = C_Initialize(NULL_PTR);
+	rv = CRYPTOKI_F_PTR( C_Initialize(NULL_PTR) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
 
 	// Open read-only session
-	rv = C_OpenSession(SLOT_INIT_TOKEN, CKF_SERIAL_SESSION, NULL_PTR, NULL_PTR, &hSessionRO);
+	rv = CRYPTOKI_F_PTR( C_OpenSession(m_initializedTokenSlotID, CKF_SERIAL_SESSION, NULL_PTR, NULL_PTR, &hSessionRO) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
 
 	// Open read-write session
-	rv = C_OpenSession(SLOT_INIT_TOKEN, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSessionRW);
+	rv = CRYPTOKI_F_PTR( C_OpenSession(m_initializedTokenSlotID, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSessionRW) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
 
 	// Login USER into the sessions so we can create private objects
-	rv = C_Login(hSessionRO,CKU_USER,pin,pinLength);
+	rv = CRYPTOKI_F_PTR( C_Login(hSessionRO,CKU_USER,m_userPin1,m_userPin1Length) );
 	CPPUNIT_ASSERT(rv == CKR_OK);
 
 	// Generate base key
